;********************************************************
;
;               LED Scan control for RCC
;
;                                 Device : PIC16F628
;                                 Clock  : 4MHz(INT)
;                                 Author : Seiichi Inoue
;********************************************************

        list            p=pic16f628
        include         p16f628.inc
        __config        h'3F10' ;OSC is Int 4MHz
                                ;RA5,RA6,RA7,RB4 are I/O
                                ;Power-up timer ON
                                ;Code protection OFF
                                ;Data code protection OFF
                                ;Brown-out detection OFF
                                ;Watchdog timer OFF
        errorlevel      -302    ;Eliminate bank warning

;****************  Label Definition  ********************
        cblock  h'20'
t1000y                          ;1000th of year
t100y                           ;100th of year
t10y                            ;10th of year
t1y                             ;1st of year
t10mo                           ;10th of month
t1mo                            ;1st of month
t10d                            ;10th of day
t1d                             ;1st of day

t10h                            ;10th of hour
t1h                             ;1st of hour
t10m                            ;10th of minute
t1m                             ;1st of minute
t10s                            ;10th of second
t1s                             ;1st of second
ra_index                        ;Index for RA port
rb_index                        ;Index for RB port
        endc

ra_adr  equ    t10h             ;RA scan data table adr
d10h    equ    b'00000001'      ;10th of hour
d1h     equ    b'00000010'      ;1st of hour
d10m    equ    b'00000100'      ;10th of minute
d1m     equ    b'00001000'      ;1st of minute
d10s    equ    b'01000000'      ;10th of second
d1s     equ    b'10000000'      ;1st of second

rb_adr  equ    t1000y           ;RB scan data table adr
d1000y  equ    b'11111110'      ;1000th of year
d100y   equ    b'11111101'      ;100th of year
d10y    equ    b'11111011'      ;10th of year
d1y     equ    b'11110111'      ;1st of year
d10mo   equ    b'11101111'      ;10th of month
d1mo    equ    b'11011111'      ;1st of month
d10d    equ    b'10111111'      ;10th of day
d1d     equ    b'01111111'      ;1st of day

;***************** Program Start ************************
        org     0               ;Reset Vector
        goto    init
        org     4               ;Interrupt Vector
        goto    int

;******************* Initial Process ********************
init
;*** Set Port mode
        bsf     status,rp0      ;Change to Bank1
        movlw   b'00000000'     ;RA7-0:OUT mode
        movwf   trisa           ;Set TRISA reg
        movlw   b'00000000'     ;RB7-0:OUT mode
        movwf   trisb           ;Set TRISB reg

;*** Set Option reg
        movlw   b'00000001'     ;PS=1:4
        movwf   option_reg      ;Set OPTION_REG

;*** Set PIE1 reg
        movlw   b'00000000'     ;TMR1IE=OFF
        movwf   pie1            ;Set PIE1 reg
        bcf     status,rp0      ;Change to Bank0

;*** Set CMCON reg
        movlw   b'00000111'     ;RA port to digital
        movwf   cmcon           ;Set CMCON reg

;*** Set TMR0 reg
        movlw   d'6'            ;256-1000us/4us=6
        movwf   tmr0            ;Set TMR0

;*** Other registers and work area initialization
        clrf    porta           ;Clear PORTA
        clrf    portb           ;Clear PORTB
        clrf    ra_index        ;Clear PORTA index
        clrf    rb_index        ;Clear PORTB index
        movlw   d1000y          ;Read 1000th of year data
        movwf   t1000y          ;Set 1000th of year data
        movlw   d100y           ;Read 100th of year data
        movwf   t100y           ;Set 100th of year data
        movlw   d10y            ;Read 10th of year data
        movwf   t10y            ;Set 10th of year data
        movlw   d1y             ;Read 1st of year data
        movwf   t1y             ;Set 1st of year data
        movlw   d10mo           ;Read 10th of month data
        movwf   t10mo           ;Set 10th of month data
        movlw   d1mo            ;Read 1st of month data
        movwf   t1mo            ;Set 1st of month data
        movlw   d10d            ;Read 10th of day data
        movwf   t10d            ;Set 10th of day data
        movlw   d1d             ;Read 1st of day data
        movwf   t1d             ;Set 1st of day data
        movlw   d10h            ;Read 10th of hour data
        movwf   t10h            ;Set 10th of hour data
        movlw   d1h             ;Read 1st of hour data
        movwf   t1h             ;Set 1st of hour data
        movlw   d10m            ;Read 10th of minute data
        movwf   t10m            ;Set 10th of minute data
        movlw   d1m             ;Read 1st of minute data
        movwf   t1m             ;Set 1st of minute data
        movlw   d10s            ;Read 10th of second data
        movwf   t10s            ;Set 10th of second data
        movlw   d1s             ;Read 1st of second data
        movwf   t1s             ;Set 1st of second data

;*** Set interrupt condition
        movlw   b'10100000'     ;GIE,TOIE=1
        movwf   intcon          ;Interruption enable

        goto    $               ;Wait interruption

int
;***** TMR0 Interruption ( 1 millisecond interval) ******
        bcf     intcon,t0if     ;Clear timer int flag
        movlw   d'6'            ;Set 1 millisecond
        movwf   tmr0            ;Set TMR0

        movlw   ra_adr          ;Read RA table head adr
        addwf   ra_index,w      ;RA table head + index
        movwf   fsr             ;Set table address
        movfw   indf            ;Read data
        movwf   porta           ;Set data to PORTA
        incf    ra_index,f      ;RA index + 1
        movfw   ra_index        ;Read RA index
        sublw   d'5'            ;Check upper limit
        btfss   status,c        ;Over ?
        clrf    ra_index        ;Yes. Clear PORTA index

        movlw   rb_adr          ;Read RB table head adr
        addwf   rb_index,w      ;RB rable head + index
        movwf   fsr             ;Set table address
        movfw   indf            ;Read data
        movwf   portb           ;Set data to PORTB
        incf    rb_index,f      ;RB index + 1
        movfw   rb_index        ;Read RB index
        sublw   d'7'            ;Check upper limit
        btfss   status,c        ;Over ?
        clrf    rb_index        ;Yes. Clear PORTB index

int_end
;************* END of interruption process **************
        retfie

;********************************************************
;          END of LED Scan control for RCC
;********************************************************

        end
